/*	
	Copyright (c) 2007 J.D. Gehman, Melbourne Australia

    This file is part of BS-REDOR.
	It was introduced by John D. Gehman, Frances Separovic, Kun Lu, and
	Anil K. Mehta (2007) in J. Phys. Chem. B 111, pp7802-7811

    BS-REDOR is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    BS-REDOR is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _PAAR_INCLUDED_

#include <stdio.h>
#include <cstring>
#include <fstream>
#include <iomanip>
#include <cstdlib>
#include <cmath>
#include <ctime>
#include <iostream>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_multiroots.h>

#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_multifit_nlin.h>
#include <gsl/gsl_deriv.h>
#include <gsl/gsl_min.h>
#include <gsl/gsl_errno.h>

#include <gsl/gsl_math.h>
#include <gsl/gsl_diff.h>
#include <gsl/gsl_eigen.h>

#include <gsl/gsl_linalg.h>
using namespace std;
const double VERSION = 1.0;

const double gC = 6.7283;           // E7 rad/T/s
const double gN = 2.712;
const double gD = 4.1066;
const double gF = 25.181;
const double gP = 10.841;
const double gH = 26.7522205;
const double hbar = 1.05457168;         // E-34 Js
const double pi = 3.14159265359;
const double pi2 = 6.28318530718;

#define _PAAR_INCLUDED_
#endif

class Paar {
public:
    int np;     // number of discrete points for distance distribution
    int nc;     // number of data points
	double chi2;
	char mode[4];

	Paar( const char * );
	~Paar();
	Paar( const Paar& );
	double calc_state();
	double calc_state_dataonly_nosim();
	int copy_min( const Paar& );
	int copy_max( const Paar& );
	int write_out();
	int gnuplot_out();
	int calc_dist_stats();
	void set_lagrange(int, double);
	
private:
	friend class BoltzmannME;
    char xN[6]; // char array for detected nucleus
    char yN[6]; // char array for dephasing nucleus
    double gX;  // gamma of detect nucleus
    double gY;  // gamma of dephasing nucleus
    double near;    // closest approach of nuclei in distance distribution
    double far;     // farthest reach of distance distribution
    double grain;   // linear increment size (in Angstroms) of distance distr.
    int pp2pi;  // points per 2pi radians to digitally sample
    double t1;      // time in us to middle of mid-cycle pulse
    double MAS;     // spin rate in Hz

    // Distance Model variables
    int adjnp;
    int adjnc;
    gsl_vector *density;    // array of normalized densities of each distance
                            // in distribution
    gsl_vector *adjdensity; // array of normalized densities of each distance
                            // in distribution
    gsl_matrix *dephase_by_cycles;  // dim nc x np 
									// matrix which converts distance
                                    // distribution to data
    gsl_matrix *dephase_by_2;
    gsl_matrix *dephase_by_2sm;
    gsl_vector *SmoothME;

    // Dipolar Coupling 
    double DDConstant;  // Dipolar coupling constance sans distance attenuation
    gsl_vector *distances;
    gsl_vector *adjdists;
    gsl_vector *dd;     // dipolar coupling constants corresponding to *dists

    // REDOR experimental and analysis parameters 
    gsl_vector *cycles;     // array of values for number of cycles
    gsl_vector *S;          // array of experimental rdflag='y' intensities 
    gsl_vector *Snot;       // array of experimental rdflag='n' intensities
    gsl_vector *redor;      // array of experimental REDOR curve values
    gsl_vector *error;      // array of experimental REDOR error values
    gsl_vector *sqrerr;     // array of experimental REDOR sqrt(error) values
    gsl_vector *SoverSnot;  // dim nc array of calculated REDOR curve values

    gsl_vector *residualy;

    gsl_vector *adjust;     // adjustments to data for prior knowledge
	gsl_vector *adjust_by_cycles;	// dim nc
    gsl_vector *avg;        // avg value n_i after MC error analysis
    gsl_vector *stdev;      // stdev of n_i after MC error analysis

    gsl_vector *lagrange;

    double scale, scaling1, scaling2;
	int lastpoint;

    double rmsd;
    double oldrmsd;
    double avgdev;
    double totaladjustdensity;

    // MC error statistics
    gsl_vector *mcavg;
    gsl_vector *mcavgerr;
    gsl_vector *mcstdev;
    gsl_vector *mcstdeverr;
	char SoSname[79], distname[79], outbase[79], saname[79], str[320];
	char lambdaDname[79];
	ofstream lfile;
	string buf;			// generic string class variable
	char path[179];
	int ntries, erriterT, permcblock, maxmcblocks;
//	int iterT;
//	double maxstep, boltzmann, inittemp, attentemp, mintemp;
	double errmaxstep, errboltzmann, errinittemp, errattentemp; 
	double errmintemp, minstdev2, maxmcrmsd;
	time_t seconds;

	gsl_vector *iterT;
	gsl_vector *inittemp;
	gsl_vector *attentemp;
	gsl_vector *boltzmann;
	gsl_vector *maxstep;
	gsl_vector *mintemp;
};

